.fw-overview-menu-wrapper {
    position: relative;
    background-color: var(--cs-white-background);
    padding: 11px 8px;
}

#fw-overview-menu {
    position: relative;
    margin: 0 213px;
}

#fw-overview-menu::after {
    content: '';
    display: table;
    clear: both;
}

@media (max-width: 1429px) {
    #fw-overview-menu {
        margin: 0 auto;
        max-width: 1004px;
    }
}

.fw-overview-menu-item {
    position: relative;
    float: left;
    margin-right: 14px;
}

.fw-overview-menu-item.search {
    float: right;
    margin-right: 0;
}

.fw-overview-menu-item.search div.fw-button.disabled {
    background-color: var(--input-background-color);
}

.fw-overview-menu-item.search div.fw-button.disabled:focus-within {
    background-color: var(--input-focus-background-color);
}

.fw-text-menu {
    height: 40px;
    color: var(--cs-white-text-secondary);
    padding: 0;
    border: 0 none;
    background-color: transparent;
    font-size: 14px;
    cursor: pointer;
}

.fw-text-menu:hover {
    color: var(--cs-white-text);
}

.fw-dropdown-menu {
    padding-top: 14px;
    font-size: 14px;
    color: var(--cs-white-text-secondary);
    cursor: pointer;
    height: 40px;
    box-sizing: border-box;
}

.fw-dropdown-menu:hover {
    color: var(--cs-white-text);
}

.fw-dropdown-menu label {
    cursor: pointer;
}

div.select-action {
    cursor: pointer;
    min-width: 38px;
}

div.select-action input[type='checkbox'] {
    cursor: default;
}

span.select-action-dropdown {
    margin-left: 5px;
}

.fw-button input[type='text'] {
    width: calc(100% - 30px);
    height: 30px;
    border: 0;
}

.fw-white input[type='text'] {
    color: var(--cs-light-text);
    background-color: var(--cs-light-background);
}

.fw-button.fw-white:focus-within {
    background-color: var(--input-focus-background-color);
}

.fw-button.fw-white input[type='text']:focus,
.fw-white.fw-input:hover {
    background-color: var(--input-focus-background-color);
}
