# Generated by Django 3.1.4 on 2021-04-02 20:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("user", "0004_auto_20210402_2049"),
    ]

    operations = [
        migrations.DeleteModel(
            name="TeamMember",
        ),
    ]
