{% extends "email_base.html" %}
{% load account %}
{% load i18n %}

{% block content %}
<h1>
    {% blocktrans with site_name=current_site.name %}Hello from {{ site_name }}!{% endblocktrans %}
</h1>

<p>
    {% blocktrans %}You are receiving this e-mail because you or someone else has requested a password for your user account.{% endblocktrans %}
</p>
<p>
    {% blocktrans %}It can be safely ignored if you did not request a password reset. Click the link below to reset your password.{% endblocktrans %}
</p>

<p><a href="{{ password_reset_url }}">{{ password_reset_url }}</a></p>

{% if username %}
<p>
{% blocktrans %}In case you forgot, your username is {{ username }}.{% endblocktrans %}
</p>
{% endif %}

<p>
    {% blocktrans with site_name=current_site.name %}Thank you from {{ site_name }}!{% endblocktrans %}
</p>
<p>{{ current_site.domain }}</p>
{% endblock %}
