# Generated by Django 4.1.2 on 2023-01-04 16:02

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("website", "0007_publication_abstract"),
    ]

    operations = [
        migrations.AlterField(
            model_name="editor",
            name="user",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="website_editor",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]
