:root {
    overflow: hidden;
}

body.cms {
    background-color: white;
    padding: 30px;
    font-size: 14pt;
    min-width: initial;
    overflow-y: auto;
    max-height: 100vh;
    width: initial;
    height: initial;
}

body.cms div.article-title {
    font-size: 25pt;
}

body.cms h1,
body.cms h2,
body.cms h3,
body.cms h4,
body.cms h5,
body.cms h6,
body.cms p {
    padding-top: 10px;
}

body.cms h1 {
    font-size: 20pt;
}

body.cms div.headers h1.site-name {
    font-family: Roboto, sans-serif;
    font-size: 65px;
    font-weight: 650;
    text-align: center;
}

body.cms div.content {
    margin-top: 48px;
}


body.cms div.articles {
    width: 80vw;
    background-color: white;
    display: inline-block;
}

body.cms div.filters {
    width: 15vw;
    background-color: white;
    display: inline-block;
    vertical-align: top;
    color: #1f1e1f;
    font-size: 16px;
    margin-top: 31px;
    margin-right: 4vh;
    position: sticky;
    top: 10px;
}

body.cms div.filters div.filter {
    margin-bottom: 16px;
}

body.cms div.filters div.filter h3.filter-title {
    font-weight: 650;
    border-bottom: 2px dotted #1f1e1f;
}

body.cms div.filters div.filter span.keyword,
body.cms div.filters div.filter span.author {
    display: block;
    padding: 4px 0;
    border-bottom: 1px solid #cecece;
    cursor: pointer;
}

body.cms div.filters div.filter span.keyword.selected,
body.cms div.filters div.filter span.author.selected {
    background-color: #1f1e1f;
    color: white;
}



body.cms div.articles a.article {
    color: black;
    border-bottom: 1px dotted #cecece;
    padding: 10px;
    display: block;
}

body.cms div.articles a.article:hover {
    background-color: #cecece;
}


body.cms div.keywords div.keyword {
    display: inline-block;
    height: 25px;
    border-radius: 4px;
    background-color: #84b4a7;
    font-family: Roboto, sans-serif;
    color: #1f1e1f;
    font-size: 12px;
    padding: 10px 10px 0;
    margin-right: 10px;
    margin-top: 4px;
    font-weight: bold;
}

body.cms div.articles a.article h1.article-title {
    font-family: Roboto, sans-serif;
    font-size: 18px;
    font-weight: bold;
}

body.cms div.articles a.article h3.article-updated,
body.cms div.articles a.article div.authors {
    font-family: Roboto, sans-serif;
    font-size: 14px;
    font-style: italic;
    display: inline-block;
}

body.cms div.articles a.article div.authors div.author {
    display: inline-block;
}

body.cms div.articles a.article div.authors div.author + div.author::before {
    content: ', ';
}


body.cms div.abstract {
    font-size: 13px;
}
