# Generated by Django 3.2.13 on 2022-07-07 17:55

from django.db import migrations, models
import django.db.models.deletion
import website.models


class Migration(migrations.Migration):

    dependencies = [
        ("website", "0002_auto_20220703_1201"),
    ]

    operations = [
        migrations.AddField(
            model_name="publication",
            name="html_output",
            field=models.TextField(default=""),
        ),
        migrations.AddField(
            model_name="publication",
            name="html_src",
            field=models.TextField(default=""),
        ),
        migrations.CreateModel(
            name="PublicationAsset",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "file",
                    models.FileField(
                        help_text="A file references in the HTML of a Publication. The filepath will be replaced with the final url of the file in the style.",
                        upload_to=website.models.publication_asset_location,
                    ),
                ),
                (
                    "filepath",
                    models.CharField(
                        help_text="The original filepath.",
                        max_length=255,
                    ),
                ),
                (
                    "publication",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="website.publication",
                    ),
                ),
            ],
        ),
    ]
