# Generated by Django 4.0.4 on 2022-05-13 20:03

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user", "0008_auto_20210604_0758"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="contacts",
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL),
        ),
    ]
