.fw-ar-container {
    display: inline-block;
    width: 338px;
    height: 360px;
    background-color: var(--cs-white-background);
    border: solid 1px var(--cs-white-border);
    vertical-align: middle;
    padding: 20px 14px 0;
    overflow-y: auto;
    margin-bottom: 16px;
}

.fw-ar-button {
    display: inline-block;
    margin: 0 16px;
    vertical-align: middle;
    cursor: pointer;
}
