# Generated by Django 3.2.13 on 2022-07-22 17:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("website", "0006_auto_20220715_1350"),
    ]

    operations = [
        migrations.AddField(
            model_name="publication",
            name="abstract",
            field=models.TextField(default=""),
        ),
    ]
