# Generated by Django 3.2.13 on 2022-07-15 08:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("website", "0004_publication_title"),
    ]

    operations = [
        migrations.AddField(
            model_name="publication",
            name="authors",
            field=models.JSONField(default=list),
        ),
        migrations.AddField(
            model_name="publication",
            name="keywords",
            field=models.JSONField(default=list),
        ),
    ]
